# Copyright (C) 2023, NG:ITL
import logging
import tempfile
import unittest
import re
import abc

from pathlib import Path

from ngitl_common_py.log import init_logging, init_emergency_logging, get_log_filepath, flush


class LogTest(unittest.TestCase):

    TEST_COMPONENT_NAME = "unittest"

    def setUp(self) -> None:
        self.tmp_dir = Path(tempfile.mkdtemp())
        self.init_log()

    @abc.abstractmethod
    def init_log(self):
        raise NotImplementedError

    # def find_log_file(self) -> Optional[Path]:
    #     elements = os.listdir(self.tmp_dir)
    #     for element in elements:
    #         if re.search(r"(\d{8}-\d{6}_|)unittest(_emergency|)\.log", element):
    #             return self.tmp_dir / element

    def log_test_lines(self):
        logging.debug("log line 1")
        logging.info("log line 2")
        logging.warning("log line 3")
        logging.error("log line 4")

    def assert_log_line(self, expected_line_as_regex: str):
        flush()
        with open(get_log_filepath(), "r") as log_file:
            lines = log_file.readlines()
            for line in lines:
                if re.search(expected_line_as_regex, line):
                    return
            self.fail(f"Unable to find expected line in log file: \"{expected_line_as_regex}\"")

    def assert_test_log_lines(self):
        self.assert_log_line(r"\[DEBUG\]\[test_log\]: log line 1$")
        self.assert_log_line(r"\[INFO\]\[test_log\]: log line 2$")
        self.assert_log_line(r"\[WARNING\]\[test_log\]: log line 3$")
        self.assert_log_line(r"\[ERROR\]\[test_log\]: log line 4$")

class RegularLogTest(LogTest):

    def init_log(self):
        init_logging(self.TEST_COMPONENT_NAME, self.tmp_dir, "DEBUG")

    def assert_regular_welcome_log_line(self):
        self.assert_log_line("Logging initialized!")

    def test_RegularLogInitialized_PrintLogLines_ExpectedLinesLogged(self) -> None:
        self.log_test_lines()
        self.assert_regular_welcome_log_line()
        self.assert_test_log_lines()


class EmergencyLogTest(LogTest):

    def init_log(self):
        init_emergency_logging(self.TEST_COMPONENT_NAME, log_file_directory=self.tmp_dir)

    def assert_emergency_welcome_log_line(self):
        self.assert_log_line("\[ERROR\]\[log\]: Emergency log initialized!$")

    def test_RegularLogInitialized_PrintLogLines_ExpectedLinesLogged(self) -> None:
        self.log_test_lines()
        self.assert_emergency_welcome_log_line()
        self.assert_test_log_lines()






# Copyright (C) 2023, NG:ITL
import os
import tempfile
import unittest
from pathlib import Path

from ngitl_common_py import autostart


class WindowsAutostartTest(unittest.TestCase):

    def setUp(self) -> None:
        tmp_dir = tempfile.mkdtemp()
        autostart_dir = os.path.join(tmp_dir, 'autostart')
        os.mkdir(autostart_dir)
        self.test_link_file = Path(os.path.join(autostart_dir, 'test.lnk'))
        self.test_target_file = Path(__file__)

    def enable_autostart(self):
        autostart.activate_autostart(self.test_target_file, self.test_link_file)

    def disable_autostart(self):
        autostart.deactivate_autostart(self.test_link_file)

    def test_AutostartEnabled_IsAutostartIsEnabled_ReturnTrue(self):
        self.enable_autostart()
        self.assertTrue(autostart.is_autostart_enabled(self.test_link_file))

    def test_AutostartDisabled_IsAutostartIsEnabled_ReturnFalse(self):
        self.disable_autostart()
        self.assertFalse(autostart.is_autostart_enabled(self.test_link_file))

    def test_AutostartEnabled_IsLinkFileExisting_FileExisting(self):
        self.enable_autostart()
        self.assertTrue(os.path.isfile(self.test_link_file))

    def test_AutostartDisabled_IsLinkFileExisting_FileNotExisting(self):
        self.disable_autostart()
        self.assertFalse(os.path.isfile(self.test_link_file))



# Copyright (C) 2023, NG:ITL
import os.path
import subprocess
from pathlib import Path


NOTEPAD_EXECUTABLE_PATH = Path(r"C:\Windows\System32\notepad.exe")
NOTEPADPP_EXECUTABLE_PATH = Path(r"C:\Program Files (x86)\Notepad++\notepad++.exe")


def _is_notepadpp_available() -> bool:
    return os.path.isfile(NOTEPADPP_EXECUTABLE_PATH)


def _open_file_with_notepadpp(filepath: Path) -> None:
    subprocess.Popen(
        [NOTEPADPP_EXECUTABLE_PATH, "-nosession", filepath],
        creationflags=subprocess.DETACHED_PROCESS,
    )


def _open_file_with_notepad(filepath: Path) -> None:
    subprocess.Popen([NOTEPAD_EXECUTABLE_PATH, filepath], creationflags=subprocess.DETACHED_PROCESS)


def open_file_viewer(filepath: Path) -> None:
    if _is_notepadpp_available():
        _open_file_with_notepadpp(filepath)
    else:
        _open_file_with_notepad(filepath)



# Copyright (C) 2023, NG:ITL
import logging
import datetime
import sys
from pathlib import Path
from typing import Optional

LOG_LEVELS = {"DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"}
LOG_FORMAT = "%(asctime)s [%(levelname)s][%(module)s]: %(message)s"
EMERGENCY_LOGGING_LOG_LEVEL = "DEBUG"

log_filepath: Optional[Path] = None


def init_logging(component_name: str, log_file_directory: Path, logging_level: str):
    global log_filepath
    timestamp = datetime.datetime.now()
    timestamp_prefix = timestamp.strftime("%Y%m%d-%H%M%S")
    log_filepath = log_file_directory / f"{timestamp_prefix}_{component_name}.log"
    stdout_handler = logging.StreamHandler(sys.stdout)
    file_handler = logging.FileHandler(log_filepath)
    logging.basicConfig(
        force=True,
        handlers=[stdout_handler, file_handler],
        level=logging_level,
        format=LOG_FORMAT,
    )
    logging.info("Logging initialized!")


def init_emergency_logging(component_name: str, log_file_directory: Path = Path.cwd()):
    global log_filepath
    log_filepath = log_file_directory / f"{component_name}_emergency.log"
    stdout_handler = logging.StreamHandler(sys.stdout)
    file_handler = logging.FileHandler(log_filepath)
    logging.basicConfig(
        force=True,
        handlers=[stdout_handler, file_handler],
        level=EMERGENCY_LOGGING_LOG_LEVEL,
        format=LOG_FORMAT,
    )
    logging.error("Emergency log initialized!")


def validate_log_level(log_level: str) -> bool:
    return log_level in LOG_LEVELS


def get_log_filepath() -> Path:
    global log_filepath
    return log_filepath


def flush():
    logger = logging.getLogger()
    for handler in logger.handlers:
        handler.flush()



#!/usr/bin/env python
# Copyright (C) 2022, NG:ITL
import versioneer
from setuptools import find_packages, setup


setup(
    name="ngitl_common_py",
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description="Helper classes and functions for common python problems",
    author="NG:ITL",
    license="GPLv3",
    author_email="torsten.wylegala@volkswagen.de",
    url="https://github.com/vw-wob-it-edu-ngitl/ngitl_common_py/",
    packages=find_packages("."),
    install_requires=["pywin32==306"],
)
